/// <summary>
/// Clock: the simple application in C# illustrating
/// the drawing in the form window and the using of threads.
/// It draws a clock face with moving arrows
/// and shows also the text representation of current time.
/// The thread is used to refresh the picture every second.
///          Author: V.Borisenko     vvb@mech.math.msu.su
/// </summary>

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

namespace Hello
{
    /// <summary>
    /// The simple Clock
    /// </summary>
    public class Form1 : System.Windows.Forms.Form
    {
        private System.Windows.Forms.Label label1;
        private System.ComponentModel.IContainer components;

        private Thread refreshThread = null;
        private bool stopRefreshThread = false;

        // Clock face geometry
        const int clock_w  = 160;
        const int clock_h  = 160;
        const int clock_dx = 10;
        const int clock_dy = 10;

        // Cosines, sinuses used to draw the clock face
        double[] cosines, sinuses;

        // Pens used for drawing
        Pen blackPen = new Pen(Color.Black, 2);
        Pen bluePen = new Pen(Color.Blue, 1);
        Pen redPen = new Pen(Color.Red, 1);

        public Form1()
        {
            //
            // Required for Windows Form Designer support
            //
            InitializeComponent();

            //
            // TODO: Add any constructor code after InitializeComponent call
            //

            // For clock face drawing: compute cos, sin of 5-minute angles
            cosines = new double[12]; sinuses = new double[12];
            double angle = 0;
            double da = Math.PI / 6.0;
            for (int i = 0; i < 12; i++, angle += da) {
                cosines[i] = Math.Cos(angle);
                sinuses[i] = Math.Sin(angle);
            }

            // Create and start the refreshing thread
            refreshThread = new Thread(
                new ThreadStart(this.RunRefreshThread)
            );
            refreshThread.Start();
        }

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                if (components != null)
                {
                    components.Dispose();
                }
            }
            base.Dispose(disposing);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.label1 = new System.Windows.Forms.Label();
            this.SuspendLayout();
            //
            // label1
            //
            this.label1.BackColor = System.Drawing.Color.LightGray;
            this.label1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.label1.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
            this.label1.ForeColor = System.Drawing.Color.Black;
            this.label1.Location = new System.Drawing.Point(10, 180);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(160, 31);
            this.label1.TabIndex = 0;
            this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
            //
            // Form1
            //
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.BackColor = System.Drawing.Color.LightGray;
            this.ClientSize = new System.Drawing.Size(180, 220);
            this.Controls.AddRange(new System.Windows.Forms.Control[] {
                                                                          this.label1});
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
            this.MaximizeBox = false;
            this.Name = "Form1";
            this.Text = "Clock";
            this.ResumeLayout(false);
        }
        #endregion

        /// <summary>
        /// Drawing in the window
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            e.Graphics.Clear(Color.LightGray);

            // Draw the clock face
            e.Graphics.DrawEllipse(
                bluePen, clock_dx, clock_dy, clock_w, clock_h
            );

            int cx = clock_dx + clock_w / 2;
            int cy = clock_dy + clock_h / 2;

            // Draw 5-minute grid
            double len;
            int x, y, x1, y1;
            for (int i = 0; i < 12; i++) {
                if (i % 3 == 0)
                    len = 0.38; // Make 15-minute line a bit longer
                else
                    len = 0.44;
                x = (int)(cx + len * clock_w * cosines[i]);
                y = (int)(cy - len * clock_h * sinuses[i]);
                x1 = (int)(cx + 0.5 * clock_w * cosines[i]);
                y1 = (int)(cy - 0.5 * clock_h * sinuses[i]);
                e.Graphics.DrawLine(bluePen, x, y, x1, y1);
            }

            // Define a current time
            DateTime curtim = DateTime.Now;
            double h = (double) (curtim.Hour % 12);
            double m = (double) curtim.Minute;
            double s = (double) curtim.Second;

            // Draw the clock arrows:
            // Hour arrow
            len = 0.5 / 2.0;
            double angle = Math.PI / 2.0 -
                ((h + m / 60.0) * Math.PI / 6.0);
            x = (int)(cx + len * clock_w * Math.Cos(angle));
            y = (int)(cy - len * clock_h * Math.Sin(angle));
            e.Graphics.DrawLine(blackPen, cx, cy, x, y);

            // Minute arrow
            len = 0.75 / 2.0;
            angle = Math.PI / 2.0 -
                ((m + s / 60.0) * Math.PI / 30.0);
            x = (int)(cx + len * clock_w * Math.Cos(angle));
            y = (int)(cy - len * clock_h * Math.Sin(angle));
            e.Graphics.DrawLine(blackPen, cx, cy, x, y);

            // Seconds arrow
            len = 0.85 / 2.0;
            angle = Math.PI / 2.0 - (s * Math.PI / 30.0);
            x = (int)(cx + len * clock_w * Math.Cos(angle));
            y = (int)(cy - len * clock_h * Math.Sin(angle));
            e.Graphics.DrawLine(redPen, cx, cy, x, y);
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.Run(new Form1());
        }

        /// <summary>
        /// The starting function of the refreshing thread
        /// </summary>
        void RunRefreshThread()
        {
            while (!stopRefreshThread)
            {
                Thread.Sleep(1000);
                label1.Text = DateTime.Now.ToString();

                // Redraw the clock
                Invalidate(false);
                Update();
            }
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            stopRefreshThread = true;
            refreshThread.Join();
        }

    } // end of class Form1
} // end of namespace Hello
